--RESTORE BackupSample from our base backup.
/* --Update ModifiedDate years for future presentaions
use BackupFileGroupSample
update SALES
set ModifiedDate = dateadd(year, 1, modifieddate)
*/

USE [master];
GO
RESTORE DATABASE [BackupFileGroupSample] 
	FROM DISK = 'C:\KNOWBACKUPS\BackupFileGroupSample_BASE2.bak' WITH NORECOVERY, REPLACE;
GO

RESTORE DATABASE [BackupFileGroupSample] WITH RECOVERY;
GO

--Change database and view data in table
USE [BackupFileGroupSample];
GO
SELECT  SalesOrderID, SalesOrderDetailID, UnitPrice, ModifiedDate
FROM    Sales
WHERE   modifieddate < '2021-01-02';
GO

--Need a Full Backup then a Log backup in order to add FileGroups.
BACKUP DATABASE [BackupFileGroupSample] 
	TO DISK = 'c:\knowbackups\backups\BackupFileGroupSample_BASE.bak' WITH INIT;
GO

BACKUP LOG [BackupFileGroupSample] 
	TO DISK = 'NUL';
GO

--Create File Groups  **SHOW CURRENT FILES AND FILEGROUPS IN GUI**
USE [master];
GO

ALTER DATABASE [BACKUPFILEGROUPSAMPLE] ADD FILEGROUP [2019];
GO

ALTER DATABASE [BACKUPFILEGROUPSAMPLE] ADD FILEGROUP [2018];
GO

ALTER DATABASE [BACKUPFILEGROUPSAMPLE] ADD FILEGROUP [2017];
GO

--Create Files and add to File Groups  **SHOW FILES GROUPS CREATE - NO FILES**
USE [master];
GO

ALTER DATABASE [BackupFileGroupSample]
ADD FILE
( NAME = BackupFilegroupSample2017,
	FILENAME = 'C:\sqldata\BackupFilegroupSample2017.ndf',
	SIZE = 1MB,
	MAXSIZE = 100MB,
	FILEGROWTH = 1MB
	)TO FILEGROUP [2017];
	GO
ALTER DATABASE [BackupFileGroupSample]
ADD FILE
( NAME = BackupFilegroupSample2018,
	FILENAME = 'C:\sqldata\BackupFilegroupSample2018.ndf',
	SIZE = 1MB,
	MAXSIZE = 100MB,
	FILEGROWTH = 1MB
	)TO FILEGROUP [2018];
	GO
ALTER DATABASE [BackupFileGroupSample]
ADD FILE
( NAME = BackupFilegroupSample2019,
	FILENAME = 'C:\sqldata\BackupFilegroupSample2019.ndf',
	SIZE = 1MB,
	MAXSIZE = 100MB,
	FILEGROWTH = 1MB
	)TO FILEGROUP [2019];
	GO
	
--**SHOW FILE GROUPS, FILES, SHOW IN EXPLORER THE FILE SIZE OF 1MB**
--CREATE A PARTITION FUNCTION TO RELOCATE THE DATA
USE [BackupFilegroupSample];
GO

BEGIN TRANSACTION
CREATE PARTITION FUNCTION [SalesModifiedDateFunction](DATETIME) AS RANGE LEFT FOR VALUES (N'2018-01-01T00:00:00', N'2019-01-01T00:00:00', N'2020-01-01T00:00:00', N'2021-01-01T00:00:00')

CREATE PARTITION SCHEME [SalesModifiedDateScheme] AS PARTITION [SalesModifiedDateFunction] TO ([2017], [2018], [2019], [PRIMARY], [PRIMARY])

CREATE CLUSTERED INDEX [ClusteredIndex_on_SalesModifiedDateScheme_634683074366356000] ON [dbo].[SALES] 
(
[ModifiedDate]
)WITH (SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF) ON [SalesModifiedDateScheme]([ModifiedDate])

DROP INDEX [ClusteredIndex_on_SalesModifiedDateScheme_634683074366356000] ON [dbo].[SALES] WITH ( ONLINE = OFF )

COMMIT TRANSACTION

--@@@@@ END PARTITION FUNCTION

--Now let's work on a piecemeal restore  **** This is where things get fun ****

--Get a baseline backup of our partitioned database
BACKUP DATABASE [BackupFileGroupSample] 
	TO DISK = 'C:\KnowBackups\Backups\BackupFileGroupSample_After.bak' WITH INIT;
GO

--Backup filegroups
BACKUP DATABASE [BackupFileGroupSample]
	FILEGROUP = 'PRIMARY',
	FILEGROUP = '2017',
	FILEGROUP = '2018',
	FILEGROUP = '2019'
	TO DISK = 'C:\KNOWBACKUPS\Backups\FILEGROUP_BACKUP.BAK' WITH INIT;
GO

--View Data to show all years are available
SELECT  SalesOrderID, SalesOrderDetailID, UnitPrice, ModifiedDate
	FROM    Sales 
	WHERE modifieddate < '2021-01-01';
GO

--Restore only the primary file group.
USE [master];
GO

RESTORE DATABASE [BackupFileGroupSample] FILEGROUP = 'Primary' 
	FROM DISK = 'C:\KnowBackups\backups\BackupFileGroupSample_After.bak' WITH Partial, RECOVERY, REPLACE;
GO

--Show that only the primary file group and the transaction log are online.
SELECT  NAME, state_desc
	FROM    [BackupFileGroupSample].sys.database_files;
GO

--Can now view information in the online filegroups
SELECT  SalesOrderID, SalesOrderDetailID, UnitPrice, ModifiedDate
	FROM [BackupFileGroupSample].dbo.Sales 
	WHERE modifieddate > '2020-01-02';
GO

--Can NOT access data in the recovery_pending filegroups
SELECT  SalesOrderID, SalesOrderDetailID, UnitPrice, ModifiedDate
	FROM [BackupFileGroupSample].dbo.Sales 
	WHERE modifieddate > '2017-01-02';
GO

--RESTORE 2017 FILEGROUP
RESTORE DATABASE [BackupFileGroupSample] FILEGROUP = '2017' WITH RECOVERY;
GO

--View State
SELECT  NAME , state_desc
	FROM    [BackupFileGroupSample].sys.database_files;
GO

--QUERY 2017 Data Only
SELECT  SalesOrderID, SalesOrderDetailID, UnitPrice, ModifiedDate
	FROM [BackupFileGroupSample].dbo.Sales 
	WHERE modifieddate BETWEEN '2017-01-01' AND '2017-12-31';
GO

--QUERY More than 2017
SELECT  SalesOrderID, SalesOrderDetailID, UnitPrice, ModifiedDate
	FROM [BackupFileGroupSample].dbo.Sales 
	WHERE modifieddate > '2017-01-01';
GO

--QUERY 2017 Data and 2020 Data
SELECT  SalesOrderID, SalesOrderDetailID, UnitPrice, ModifiedDate
	FROM [BackupFileGroupSample].dbo.Sales 
	WHERE modifieddate BETWEEN '2017-01-01' AND '2017-12-31' 
		OR ModifiedDate BETWEEN '2020-01-02' AND '2020-12-31';
GO

--Restore Other File Groups
RESTORE DATABASE [BackupFileGroupSample] FILEGROUP = '2018' WITH RECOVERY;
GO

RESTORE DATABASE [BackupFileGroupSample] FILEGROUP = '2019' WITH RECOVERY;
GO

--View State
SELECT  NAME, state_desc
	FROM    [BackupFileGroupSample].sys.database_files;
GO

--View All Data
SELECT  SalesOrderID, SalesOrderDetailID, UnitPrice, ModifiedDate
	FROM [BackupFileGroupSample].dbo.Sales 
	WHERE modifieddate < '2021-01-02';
GO

USE [master]
GO


